@@ -58,15 +58,21 @@
 #include "misc.h"
 #include "pasteboard.h"
 #include "settings.h"
-#include "xfstab.h"
 #include "toolbar.h"
 
 /* if themes.mime.xml does not exist...*/
 #define FALLBACK 1
 
 
-/*#define DEBUG*/
-
+#define PIXBUF_HASH
+#ifdef PIXBUF_HASH
+static GHashTable *pixbuf_hash=NULL;
+typedef struct pixbuf_hash_t {
+    GdkPixbuf *pixbuf;
+    int size;
+    struct pixbuf_hash_t *next;
+}pixbuf_hash_t;
+#endif
 
 static GtkStyle *style;
 static gchar *icon_theme=NULL;
@@ -77,8 +83,8 @@ static const gchar *composite_type_id(const gchar *path,int type,int subtype,con
 	g_free(composite_type);
 	composite_type=g_strdup(id);
 	if (path){
-	    if (IS_DIR(type) && is_mounted(path)==1) greenball=TRUE;
-	    else if (is_in_fstab(path) && !IS_XF_FSTAB(type)) redball=TRUE;
+	    if (IS_DIR(type) && FSTAB_is_mounted(path)==1) greenball=TRUE;
+	    else if (FSTAB_is_in_fstab(path) && !IS_XF_FSTAB(type)) redball=TRUE;
 	}
 	if (IS_NOWRITE(type)||IS_XF_LNK(type)||HAS_HIDDEN(type)||greenball||redball||IS_NETREADONLY(subtype)||IS_XF_NETSHARE(subtype)){
 	    p=composite_type;
@@ -153,23 +159,113 @@ gboolean is_image (gchar *file){
     return result;
 }
 
+static
+const gchar *get_thumbnail_path(char *file, int size){
+    gchar *xdg_dir;
+    gchar *cache_dir,*filename;
+    static gchar *thumbnail_path=NULL;
+    GString *gs;
+    gchar key[11];
 
+    xdg_dir=xfce_resource_save_location (XFCE_RESOURCE_CACHE,"/",TRUE);
+    cache_dir = g_build_filename(xdg_dir,"xfce4","xffm","thumbnails",NULL);  
+    g_free(xdg_dir);
+    mkdir(cache_dir, 0770);
+    if (!g_file_test(cache_dir,G_FILE_TEST_IS_DIR)) {
+	g_free(cache_dir);
+	return NULL;
+    }
+
+    /* thumbnails are not subject to thumbnailization: */
+    filename=g_path_get_dirname(file);
+    if (strcmp(cache_dir,filename)==0){
+	DBG("thumbnails cannot be thumbnailed:%s",file);
+	g_free(cache_dir);
+	g_free(filename);
+	return NULL;
+    }
+    g_free(filename);
+    
+    
+    gs = g_string_new(file);
+    sprintf(key, "%10u", g_string_hash(gs));
+    g_string_free(gs, TRUE);
+
+    filename=g_strdup_printf("%s-%d.png",key,size);
+    thumbnail_path = g_build_filename(cache_dir,filename,NULL);
+    g_free(filename);
+    g_free(cache_dir);
+    
+    return (const gchar *)thumbnail_path;
+ }
+
+/* some of the code in this function contributed by Thomas Leonard: */
+static
+void save_thumbnail(const gchar *thumbnail_path, const gchar *original_path, GdkPixbuf *tgt){
+	int original_width, original_height;
+	char *swidth, *sheight, *ssize, *smtime;
+	struct stat st;
+	GError *error=NULL;
+	
+	if (!thumbnail_path || !original_path) return;
+	DBG("creating thumbnail for %s (%s)",original_path,thumbnail_path);
+	
+	original_width = gdk_pixbuf_get_width(tgt);
+	original_height = gdk_pixbuf_get_height(tgt);
+
+	if (stat(original_path, &st) != 0) return;
+
+	swidth = g_strdup_printf("%d", original_width);
+	sheight = g_strdup_printf("%d", original_height);
+	ssize = g_strdup_printf("%ld", (long int)st.st_size);
+	smtime = g_strdup_printf("%ld", st.st_mtime);
+	
+	gdk_pixbuf_save(tgt,
+			thumbnail_path,
+			"png",
+			&error,
+			"tEXt::Thumb::Image::Width", swidth,
+			"tEXt::Thumb::Image::Height", sheight,
+			"tEXt::Thumb::Size", ssize,
+			"tEXt::Thumb::MTime", smtime,
+			"tEXt::Software", "Xfce4",
+			NULL);
+	if (error) g_error_free (error);
+	g_free(swidth);
+	g_free(sheight);
+	g_free(ssize);
+	g_free(smtime);
+}
+
+#define BIG_ENOUGH (96*96*3)
 G_MODULE_EXPORT
 GdkPixbuf *create_preview (char *file, int size)
 {
     GdkPixbuf *src, *tgt;
     int max_w, w, h;
-    int pw, ph;
-    double scaleW, scaleH, scale;
+    int area,pw, ph;
+    double scaleW, scaleH,scale;
     GError *error = NULL;
+    const gchar *thumbnail_path=get_thumbnail_path(file,size);
+
+    TRACE("preview %s",file);
+    
+    if (thumbnail_path && g_file_test(thumbnail_path,G_FILE_TEST_EXISTS)){
+	TRACE("retrieving thumbnail for %s (%s)",file,thumbnail_path);
+	tgt = gdk_pixbuf_new_from_file(thumbnail_path, NULL);
+	if (tgt) return tgt;
+    }
 
     switch (size)
     {
 	default:
+	    max_w = w = 96, h = 48;
+	    break;
+	case SMALL:
 	    max_w = w = 48, h = 24;
 	    break;
 	case MEDIUM:
-	    max_w = w = 68, h = 34;
+	    max_w = w = 100, h = 50;
 	    break;
 	case BIG:
 	    max_w = w = 100, h = 50;
@@ -178,61 +274,35 @@ GdkPixbuf *create_preview (char *file, int size)
 	    max_w = w = 200, h = 100;
 	    break;
     }
-#if 0
-#ifdef HAVE_LIBRSVG
-    if (strstr(file,".svg")) {
-	
-	src = rsvg_pixbuf_from_file ((const gchar *)file,&error);
-	if (error) {
-	    /*g_warning("%s: %s",file,error->message);*/
-	    g_error_free(error);
-	    error=NULL;
-	}
-	/******************/
-	/* FIXME: this block of code repeats itself a few lines below 
-	 * there should be a better way to get the w and h of a svg file...*/
-    ph = gdk_pixbuf_get_height(src);
-    pw = gdk_pixbuf_get_width(src);
-    if ((ph > h || pw > w) && pw > 0 && ph > 0){
-      scaleH = (double)h / ph;
-      scaleW = (double)w / pw;
-      if (scaleW*pw > max_w) scaleW=(double)max_w / pw;
-      scale = (scaleH < scaleW) ? scaleH : scaleW;
-      h = scale * ph;
-      w = scale * pw;
-    } else return (src);
-    if(w < 10 || h < 10)
-    {
-	if (src) g_object_unref(G_OBJECT(src));
-	return NULL;
-    }
-	/********************/
-	tgt = rsvg_pixbuf_from_file_at_size((const gchar *)file,w,h,&error);
-	if (src) g_object_unref(G_OBJECT(src));
 
-	
-	if (error){
-	    /*g_warning("%s: %s",file,error->message);*/
-	    g_error_free(error);
-	    error=NULL;
-	}
-	return tgt;
-    }
-#endif
-#endif
-    src = gdk_pixbuf_new_from_file(file, &error);
-    if(!src)
-    {
-	/*printf("DBG: pixbuf error, file=%s\n",file); */
-	return NULL;
-    }
+#if GTK_CHECK_VERSION (2,4,0)
+    src = gdk_pixbuf_new_from_file_at_size (file,w,h,&error);
     if(error){
+	g_warning(error->message);
 	g_error_free(error);
 	error=NULL;
+	return NULL;
     }
+#else /* gtk < 2.4 */
+    src = gdk_pixbuf_new_from_file(file, &error);
+    if(error || !src){
+	if (error) {
+	    g_warning("file=%s (%s)",file,error->message);
+	    g_error_free(error);
+	}
+	error=NULL;
+	return NULL;
+    }
+#endif
 
     ph = gdk_pixbuf_get_height(src);
     pw = gdk_pixbuf_get_width(src);
+    if (ph <= 0 || pw <= 0) {
+	g_object_unref(src);
+	return NULL;
+    }
+
+    area=ph*pw;
     if ((ph > h || pw > w) && pw > 0 && ph > 0){
       scaleH = (double)h / ph;
       scaleW = (double)w / pw;
@@ -240,7 +310,10 @@ GdkPixbuf *create_preview (char *file, int size)
       scale = (scaleH < scaleW) ? scaleH : scaleW;
       h = scale * ph;
       w = scale * pw;
-    } else return (src);
+    } else {
+	if (area > BIG_ENOUGH) save_thumbnail(thumbnail_path,file,src);
+	return (src);
+    }
     if(w < 10 || h < 10)
     {
 	if (src) g_object_unref(G_OBJECT(src));
@@ -251,6 +324,8 @@ GdkPixbuf *create_preview (char *file, int size)
 	return NULL;
     tgt = gdk_pixbuf_scale_simple(src, w, h, GDK_INTERP_BILINEAR);
     if (tgt) g_object_unref(G_OBJECT(src));
+
+    if (area > BIG_ENOUGH) save_thumbnail(thumbnail_path,file,tgt);
     return tgt;
 }
 G_MODULE_EXPORT
@@ -352,6 +427,77 @@ insert_pixbuf_tag (		const gchar *id,
        	x_tgt=0, y_tgt=(scale-1)*width/scale;
     else if (strcmp(where,"SE")==0) 
 	x_tgt=(scale-1)*width/scale, y_tgt=(scale-1)*width/scale;
+    else if (strcmp(where,"E")==0) 
+	x_tgt=(scale-1)*width/scale, y_tgt=(scale-1)*width/scale/2;
+
+    
+#if 0 
+    {
+	/* with this we get the tag to have a transparent part
+	 * over the icon, but where the icon *is* transparent,
+	 * and the tag as well, the resulting pixbuf will be *not*
+	 * transparent here! (It will have the color of the filled
+	 * in rectangle.*/
+	GdkBitmap *mask_return;
+	GdkPixmap *sandbox,*pixmap;
+	static GdkGC *penGC=NULL;
+	GdkColormap *cmap=gdk_colormap_get_system();
+
+	
+	GdkPixbuf *pb=gdk_pixbuf_new_subpixbuf (*tgt,
+		x_tgt,y_tgt,
+		width/scale,height/scale);
+	
+	gdk_pixbuf_render_pixmap_and_mask_for_colormap (pb, 
+		cmap, 
+		&pixmap, NULL, 128);
+	
+	sandbox = gdk_pixmap_new((GdkDrawable *)pixmap, width/scale,height/scale,-1);
+	g_object_unref(pixmap);
+
+	if (!penGC) {
+	    GdkColor pen_color= {0,65535,65535,65535};
+	    gdk_colormap_alloc_color(cmap,&pen_color,TRUE,TRUE);
+	    penGC = gdk_gc_new ((GdkDrawable *)sandbox);
+	    gdk_gc_set_colormap (penGC,cmap);
+	    gdk_gc_set_foreground (penGC, &pen_color);
+	}
+	gdk_draw_rectangle (sandbox, penGC, TRUE, 0,0,width/scale,height/scale);
+	gdk_draw_pixbuf ((GdkDrawable *)sandbox,
+                         penGC,
+                         *tgt,x_tgt,y_tgt,
+			 0,0, /* dest */
+			 width/scale,height/scale,
+                         GDK_RGB_DITHER_NONE,0,0);
+	
+	
+		
+	gdk_pixbuf_render_pixmap_and_mask_for_colormap (tag, 
+		cmap, 
+		NULL, &mask_return, 128);
+	
+	gdk_gc_set_clip_mask   (penGC,mask_return);
+
+	gdk_draw_pixbuf ((GdkDrawable *)sandbox,
+                         penGC,
+                         tag,
+                         0,0, /* src */
+			 0,0, /* dest */
+			 width/scale,height/scale,
+                         GDK_RGB_DITHER_NONE,0,0);
+	gdk_gc_set_clip_mask   (penGC,NULL);
+	
+	gdk_pixbuf_get_from_drawable (tag,
+		(GdkDrawable *)sandbox,
+                cmap,
+                0,0,0,0,
+		width/scale,height/scale);
+	g_object_unref(G_OBJECT(sandbox));
+	g_object_unref(G_OBJECT(pb));
+	g_object_unref(G_OBJECT(mask_return));
+
+    }
+#endif
     gdk_pixbuf_copy_area (tag,0,0,width/scale,height/scale,*tgt,x_tgt,y_tgt);
     g_object_unref(G_OBJECT(tag));
 }
@@ -473,11 +619,30 @@ static GdkPixbuf *icon_tell_cut(int size, const gchar *id, gboolean cut)
 {
     GtkIconSet *iconset;
     int gtksize;
+    GdkPixbuf *pixbuf;
+    
     if (!id || !strlen(id)) return NULL;
-   
+#ifdef PIXBUF_HASH
+    if (pixbuf_hash && !cut){
+	pixbuf_hash_t *pixbuf_hash_p = g_hash_table_lookup(pixbuf_hash,id);
+	if (pixbuf_hash_p){
+	    TRACE("pixbuf found for %s",id);
+	    for (;pixbuf_hash_p; pixbuf_hash_p= pixbuf_hash_p->next){
+		if (pixbuf_hash_p->size==size) break;
+	    }
+	    if (pixbuf_hash_p){
+		pixbuf=pixbuf_hash_p->pixbuf;
+		g_object_ref(pixbuf);
+		return pixbuf;
+	    } else DBG("pixbuf size %d not found",size);
+	}
+	
+    }
+#endif
 
     if (strncmp(id,"gtk-",strlen("gtk-"))==0){
-	return load_stock_icon(id ,size);
+	pixbuf=load_stock_icon(id ,size);
+	goto pixbuf_done;
 	/*return gtk_widget_render_icon(tree_details->window, id ,size, NULL);*/
     } 	
     
@@ -498,10 +663,10 @@ static GdkPixbuf *icon_tell_cut(int size, const gchar *id, gboolean cut)
 	
 	
 	if (strstr(id,"/shows_hidden")){
-	    insert_pixbuf_tag("gtk-add",&folder,size,4,"NE");
+	    insert_pixbuf_tag("gtk-add",&folder,size,4,"E");
 	}
 	else if (strstr(id,"/has_hidden")) {
-	    insert_pixbuf_tag("gtk-remove",&folder,size,4,"NE");
+	    insert_pixbuf_tag("gtk-remove",&folder,size,4,"E");
 	}
 	
 	if (strstr(id,"/no_write")){
@@ -519,7 +684,11 @@ static GdkPixbuf *icon_tell_cut(int size, const gchar *id, gboolean cut)
 	if (cut && folder){   
 	    iconset=gtk_icon_set_new_from_pixbuf(folder);
 	    g_object_unref(G_OBJECT(folder));
-	} else return folder;
+	} 
+	else {
+	    pixbuf=folder;
+	    goto pixbuf_done;
+	}
     }
     
     iconset = MIME_ICON_get_iconset(id,tree_details->window);
@@ -566,7 +735,39 @@ static GdkPixbuf *icon_tell_cut(int size, const gchar *id, gboolean cut)
 	    /*gtksize=GTK_ICON_SIZE_SMALL_TOOLBAR; break;*/
     }
     if(!iconset) return NULL;
-    return gtk_icon_set_render_icon(iconset, style, GTK_DIR_RIGHT, (cut) ? GTK_STATE_INSENSITIVE : GTK_STATE_NORMAL, gtksize, NULL, NULL);
+    pixbuf = gtk_icon_set_render_icon(iconset, style, GTK_DIR_RIGHT, (cut) ? GTK_STATE_INSENSITIVE : GTK_STATE_NORMAL, gtksize, NULL, NULL);
+pixbuf_done:
+#ifdef PIXBUF_HASH
+    if (!cut) {
+	pixbuf_hash_t *pixbuf_hash_p,*p;
+	g_object_ref(pixbuf);
+	if (!pixbuf_hash){
+	    pixbuf_hash = g_hash_table_new(g_str_hash, g_str_equal);
+	}
+	pixbuf_hash_p = g_hash_table_lookup(pixbuf_hash,id);
+	if (!pixbuf_hash_p) {
+	    DBG("new pixbuf for %s",id);
+	    pixbuf_hash_p=(pixbuf_hash_t *)malloc(sizeof(pixbuf_hash_t));
+	    pixbuf_hash_p->size=size;
+	    pixbuf_hash_p->pixbuf=pixbuf;
+	    pixbuf_hash_p->next=NULL;
+	    g_object_ref(pixbuf);	    
+	    g_hash_table_insert(pixbuf_hash, g_strdup(id), (gpointer) pixbuf_hash_p);
+	    return pixbuf;
+	}
+	for (p=pixbuf_hash_p; p && p->next; p=p->next);
+	
+	/*if (p->size==size) g_warning("this does not happen");*/
+	
+	pixbuf_hash_p = p->next = (pixbuf_hash_t *)malloc(sizeof(pixbuf_hash_t));
+	pixbuf_hash_p->size=size;
+	pixbuf_hash_p->pixbuf=pixbuf;
+	pixbuf_hash_p->next=NULL;
+	g_object_ref(pixbuf);	    
+	return pixbuf;    
+    }
+#endif
+    return pixbuf;
 }
 
 G_MODULE_EXPORT
@@ -685,8 +886,8 @@ const gchar *resolve_icon_id (tree_entry_t * en)
   GTK_STATE_PRELIGHT,
   GTK_STATE_SELECTED,
   GTK_STATE_INSENSITIVE*/
-
-static GdkPixbuf *resolve_icon (tree_entry_t * en, tree_entry_t * p_en)
+/* use parameter for iconview size */
+static GdkPixbuf *resolve_icon (tree_entry_t * en, tree_entry_t * p_en, int s)
 {
     int gtksize, size;
     char *loc = NULL;
@@ -701,7 +902,11 @@ static GdkPixbuf *resolve_icon (tree_entry_t * en, tree_entry_t * p_en)
 
     if(!style) style = gtk_style_new();
 
-    switch (tree_details->icon_size){
+    if (s<0) {
+	gtksize = GTK_ICON_SIZE_DIALOG;
+	size = REAL_BIG;
+    }
+    else switch (tree_details->icon_size){
       case 3:
 	gtksize = GTK_ICON_SIZE_DIALOG;
 	size = REAL_BIG;
@@ -746,14 +951,14 @@ static GdkPixbuf *resolve_icon (tree_entry_t * en, tree_entry_t * p_en)
     if ( !cut && loc && IS_FILE(en->type) && !IS_NETWORK_TYPE(en->type) &&
 	 !IS_TRASH_TYPE(en->type) && !strstr(en->path, "/..Wastebasket") &&
 	 en->st->st_size <= max_preview_size*1024 && 
-	 (tree_details->preferences & IMAGE_PREVIEW ||
-	  (p_en && SHOWS_IMAGES(p_en->type)) 
+	  ((p_en && SHOWS_IMAGES(p_en->type)) 
 	 ) && (IS_IMAGE(loc))
 	)
     {
  	GdkPixbuf *tgt;
         process_pending_gtk(); 
-	tgt = create_preview(en->path, size);
+	if (s>=0) tgt = create_preview(en->path, size);
+	else tgt = create_preview(en->path, s);
 	if(tgt) return tgt;
     }
     
@@ -843,11 +1048,21 @@ void set_icon (GtkTreeModel *treemodel, GtkTreeIter * iterator)
 	return;
     /*if (en->path) printf("DBG: icon setting for %s\n",en->path); */
     set_font(treemodel,iterator); 
-    Icon = resolve_icon(en, p_en);
+    Icon = resolve_icon(en, p_en,0);
     update_iconcolumn_for_row(treemodel, iterator,Icon);	
     if (Icon) g_object_unref (G_OBJECT (Icon));    
 }
 
+G_MODULE_EXPORT
+GdkPixbuf *get_icon (tree_entry_t *en,tree_entry_t *p_en)
+{
+    GdkPixbuf *Icon;
+    if (!en) return NULL;  
+    TRACE("shows images=%d",SHOWS_IMAGES(p_en->type)); 
+    Icon = resolve_icon(en, p_en,-1);
+    return Icon;
+}
+
 G_MODULE_EXPORT
 void update_icon (GtkTreeModel *treemodel, GtkTreeIter * iterator)
 {
@@ -875,7 +1090,7 @@ void update_icon (GtkTreeModel *treemodel, GtkTreeIter * iterator)
 	{
 	    memcpy(en->st, &st, sizeof(struct stat));
 	    en->count = count_files(en->path);
-	    Icon = resolve_icon(en, p_en);
+	    Icon = resolve_icon(en, p_en,0);
 	    D(printf(".");fflush(NULL);)
 	    update_iconcolumn_for_row(treemodel, iterator,Icon);	
 	    update_text_cell_for_row(SIZE_COLUMN,treemodel, iterator,sizetag((off_t)-1, en->count)); 
@@ -888,7 +1103,7 @@ void update_icon (GtkTreeModel *treemodel, GtkTreeIter * iterator)
     if(IS_CUT(en->type) || in_pasteboard(en)==2)
     {
 	/*printf("dbg update icon %s\n",en->path); */
-	Icon = resolve_icon(en, p_en);
+	Icon = resolve_icon(en, p_en,0);
         update_iconcolumn_for_row(treemodel, iterator,Icon);	
 	if (Icon) g_object_unref (G_OBJECT (Icon));    
 	if(in_pasteboard(en)==2){
@@ -963,7 +1178,7 @@ GdkPixbuf *load_stock_icon (const gchar *id, int size){
     /*gchar *s,*p=NULL;*/
     GdkPixbuf *pixbuf=NULL;
     if (strncmp(id,"gtk-",strlen("gtk-"))!=0){
-	g_warning("load_stock_icon called on non stock item %d",id);
+	g_warning("load_stock_icon called on non stock item %s",id);
 	return NULL;
     }
 #if 0
