@@ -1,4 +1,4 @@
-/* $Id: xfce-resource.c,v 1.6 2004/08/04 23:16:29 olivier Exp $ */
+/* $Id: xfce-resource.c 20955 2006-04-15 22:46:01Z benny $ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -87,7 +87,10 @@ _res_split_and_append (const gchar     *dir_list,
   dirs = g_strsplit (dir_list, ":", -1);
   for (n = 0; dirs[n] != NULL; ++n)
     {
-      _list[type] = g_list_append (_list[type], dirs[n]);
+      if (g_path_is_absolute (dirs[n]))
+        _list[type] = g_list_append (_list[type], dirs[n]);
+      else
+        g_free (dirs[n]);
     }
   g_free (dirs);
 }
@@ -174,12 +177,14 @@ _res_init (void)
    */
   dirs = _res_getenv ("XDG_DATA_DIRS", DEFAULT_XDG_DATA_DIRS);
   _res_split_and_append (dirs, XFCE_RESOURCE_DATA);
+  _res_split_and_append (DATADIR, XFCE_RESOURCE_DATA);
 
   /*
    * Config dirs
    */
   dirs = _res_getenv ("XDG_CONFIG_DIRS", DEFAULT_XDG_CONFIG_DIRS);
   _res_split_and_append (dirs, XFCE_RESOURCE_CONFIG);
+  _res_split_and_append (SYSCONFDIR "/xdg", XFCE_RESOURCE_CONFIG);
 
   /*
    * Icons dirs
