@@ -62,6 +62,12 @@ typedef struct _NetkTaskClass NetkTaskClass;
 
 #define N_SCREEN_CONNECTIONS 5
 
+#ifdef PANGO_ELLIPSIZE_END
+#define NETK_ELLIPSIZE_END PANGO_ELLIPSIZE_END
+#else
+#define NETK_ELLIPSIZE_END 3
+#endif
+
 typedef enum
 {
     NETK_TASK_CLASS_GROUP,
@@ -126,9 +132,6 @@ struct _NetkTasklistPrivate
     /* Calculated by update_lists */
     GList *windows;
     GList *class_groups;
-    /* This is needed so we don't lose track of windows that have the
-     * _NET_WM_STATE_SKIP_TASKBAR hint set. */
-    GList *hidden_windows;
 
     GHashTable *win_hash;
     GHashTable *class_group_hash;
@@ -340,7 +343,7 @@ netk_task_finalize (GObject * object)
 
     if (task->menu)
     {
-        gtk_widget_destroy (task->menu);
+        g_object_unref (task->menu);
         task->menu = NULL;
     }
 
@@ -798,7 +801,10 @@ netk_tasklist_size_request (GtkWidget * widget, GtkRequisition * requisition)
         l = l->next;
     }
 
-    max_width = DEFAULT_BUTTON_WIDTH;
+    if (tasklist->priv->show_label)
+        max_width = DEFAULT_BUTTON_WIDTH;
+    else
+        max_width = max_height;
     
     tasklist->priv->max_button_width = max_width;
     tasklist->priv->max_button_height = max_height;
@@ -1236,22 +1242,6 @@ netk_tasklist_new (NetkScreen * screen)
     return GTK_WIDGET (tasklist);
 }
 
-static void
-netk_tasklist_free_tasks_data (NetkTask *task)
-{
-    if (task->blink_timeout_id)
-    {
-        g_source_remove (task->blink_timeout_id);
-        task->blink_timeout_id = 0;
-    }
-
-    /* if we just unref the task it means we lose our ref to the
-     * task before we unparent the button, which breaks stuff.
-     */
-    gtk_widget_destroy (task->button);
-}
-
-
 static void
 netk_tasklist_free_tasks (NetkTasklist * tasklist)
 {
@@ -1267,7 +1257,17 @@ netk_tasklist_free_tasks (NetkTasklist * tasklist)
         {
             NetkTask *task = NETK_TASK (l->data);
             l = l->next;
-            netk_tasklist_free_tasks_data (task);
+            
+            if (task->blink_timeout_id)
+            {
+                g_source_remove (task->blink_timeout_id);
+                task->blink_timeout_id = 0;
+            }
+
+            /* if we just unref the task it means we lose our ref to the
+             * task before we unparent the button, which breaks stuff.
+             */
+            gtk_widget_destroy (task->button);
         }
     }
     g_assert (tasklist->priv->windows == NULL);
@@ -1280,24 +1280,21 @@ netk_tasklist_free_tasks (NetkTasklist * tasklist)
         {
             NetkTask *task = NETK_TASK (l->data);
             l = l->next;
-            netk_tasklist_free_tasks_data (task);
+
+            if (task->blink_timeout_id)
+            {
+                g_source_remove (task->blink_timeout_id);
+                task->blink_timeout_id = 0;
+            }
+
+            /* if we just unref the task it means we lose our ref to the
+             * task before we unparent the button, which breaks stuff.
+             */
+            gtk_widget_destroy (task->button);
         }
     }
     g_assert (tasklist->priv->class_groups == NULL);
     g_assert (g_hash_table_size (tasklist->priv->class_group_hash) == 0);
-
-    if (tasklist->priv->hidden_windows)
-    {
-        l = tasklist->priv->hidden_windows;
-        while (l != NULL)
-        {
-            NetkTask *task = NETK_TASK (l->data);
-            l = l->next;
-            netk_tasklist_free_tasks_data (task);
-        }
-        g_list_free (tasklist->priv->hidden_windows);
-        tasklist->priv->hidden_windows = NULL;
-    }
 }
 
 /*                                                                              
@@ -1306,37 +1303,42 @@ netk_tasklist_free_tasks (NetkTasklist * tasklist)
 static gboolean                                                                 
 netk_tasklist_include_window (NetkTasklist *tasklist, NetkWindow *win)          
 {                                                                               
-    NetkWorkspace *active_workspace;
+    NetkWorkspace *active_workspace;                                              
 
-    if (tasklist->priv->include_all_workspaces)                                   
-    {
-        return TRUE;                                                                
-    }
-
-    if (netk_window_is_pinned (win))                                              
-    {
-        return TRUE;                                                                
-    }
-
-    active_workspace = netk_screen_get_active_workspace (tasklist->priv->screen); 
-
-    if (active_workspace == NULL)                                                 
-    {
-        return TRUE;                                                                
-    }
-
-    if (active_workspace != netk_window_get_workspace (win))                      
+    if (netk_window_get_state (win) & NETK_WINDOW_STATE_SKIP_TASKLIST)            
     {
         return FALSE;                                                               
     }
 
-    if (!netk_workspace_is_virtual (active_workspace))                            
+    if (tasklist->priv->include_all_workspaces)
     {
-        return TRUE;                                                                
+        return TRUE;
     }
 
-    return netk_window_is_in_viewport (win, active_workspace);                    
-}                                                                               
+    if (netk_window_is_pinned (win))
+    {
+        return TRUE;
+    }
+
+    active_workspace = netk_screen_get_active_workspace (tasklist->priv->screen); 
+
+    if (active_workspace == NULL)
+    {
+        return TRUE;
+    }
+
+    if (active_workspace != netk_window_get_workspace (win))
+    {
+        return FALSE;
+    }
+
+    if (!netk_workspace_is_virtual (active_workspace))
+    {
+        return TRUE;
+    }
+
+    return netk_window_is_in_viewport (win, active_workspace);
+}
 
 static void
 netk_tasklist_update_lists (NetkTasklist * tasklist)
@@ -1358,42 +1360,29 @@ netk_tasklist_update_lists (NetkTasklist * tasklist)
         win = NETK_WINDOW (l->data);
         if (netk_tasklist_include_window (tasklist, win))
         {
-            if (netk_window_get_state (win) & NETK_WINDOW_STATE_SKIP_TASKLIST)
+            win_task = netk_task_new_from_window (tasklist, win);
+            tasklist->priv->windows = g_list_prepend (tasklist->priv->windows, win_task);
+            g_hash_table_insert (tasklist->priv->win_hash, win, win_task);
+
+            gtk_widget_set_parent (win_task->button, GTK_WIDGET (tasklist));
+            gtk_widget_show (win_task->button);
+
+            /* Class group */
+
+            class_group = netk_window_get_class_group (win);
+            class_group_task = g_hash_table_lookup (tasklist->priv->class_group_hash, class_group);
+
+            if (class_group_task == NULL)
             {
-                win_task = netk_task_new_from_window (tasklist, win);
-                tasklist->priv->hidden_windows = 
-                    g_list_prepend (tasklist->priv->windows, win_task);
-                gtk_widget_set_parent (win_task->button, GTK_WIDGET (tasklist));
-                /* The task gotta have a button, so let's keep it, but hidden */
-                gtk_widget_hide (win_task->button);
+                class_group_task = netk_task_new_from_class_group  (tasklist, class_group);
+                gtk_widget_set_parent (class_group_task->button, GTK_WIDGET (tasklist));
+                gtk_widget_show (class_group_task->button);
+
+                tasklist->priv->class_groups = g_list_prepend (tasklist->priv->class_groups, class_group_task);
+                g_hash_table_insert (tasklist->priv->class_group_hash, class_group, class_group_task);
             }
-            else
-            {
-                win_task = netk_task_new_from_window (tasklist, win);
-                tasklist->priv->windows =
-                    g_list_prepend (tasklist->priv->windows, win_task);
-                g_hash_table_insert (tasklist->priv->win_hash, win, win_task);
 
-                gtk_widget_set_parent (win_task->button, GTK_WIDGET (tasklist));
-                gtk_widget_show (win_task->button);
-
-                /* Class group */
-
-                class_group = netk_window_get_class_group (win);
-                class_group_task = g_hash_table_lookup (tasklist->priv->class_group_hash, class_group);
-
-                if (class_group_task == NULL)
-                {
-                    class_group_task = netk_task_new_from_class_group  (tasklist, class_group);
-                    gtk_widget_set_parent (class_group_task->button, GTK_WIDGET (tasklist));
-                    gtk_widget_show (class_group_task->button);
-
-                    tasklist->priv->class_groups = g_list_prepend (tasklist->priv->class_groups, class_group_task);
-                    g_hash_table_insert (tasklist->priv->class_group_hash, class_group, class_group_task);
-                }
-
-                class_group_task->windows = g_list_prepend (class_group_task->windows, win_task);
-            }
+            class_group_task->windows = g_list_prepend (class_group_task->windows, win_task);
         }
 
         l = l->next;
@@ -1495,8 +1484,7 @@ netk_tasklist_update_icon_geometries (NetkTasklist * tasklist)
         if (!GTK_WIDGET_REALIZED (task->button))
             continue;
 
-        gdk_window_get_origin (GTK_BUTTON (task->button)->event_window, &x,
-                               &y);
+        gdk_window_get_origin (GTK_BUTTON (task->button)->event_window, &x, &y);
         width = task->button->allocation.width;
         height = task->button->allocation.height;
 
@@ -1565,45 +1553,44 @@ netk_tasklist_window_changed_workspace (NetkWindow * window,
     }
 }
 
-static gboolean                                                                 
-do_netk_tasklist_update_lists (gpointer data)                                   
-{                                                                               
-    NetkTasklist *tasklist = NETK_TASKLIST (data);                                
-    tasklist->priv->idle_callback_tag = 0;                                        
-    netk_tasklist_update_lists (tasklist);                                        
-    return FALSE;                                                                 
-}                                                                               
+static gboolean
+do_netk_tasklist_update_lists (gpointer data)
+{
+    NetkTasklist *tasklist = NETK_TASKLIST (data);
+    tasklist->priv->idle_callback_tag = 0;
+    netk_tasklist_update_lists (tasklist);
+    return FALSE;
+}
 
-static void                                                                     
-netk_tasklist_window_changed_geometry (NetkWindow   *window,                    
-                                       NetkTasklist *tasklist)                  
-{                                                                               
-    NetkTask *win_task;                                                           
-    gboolean show;                                                                
-    if (tasklist->priv->idle_callback_tag != 0)                                   
+static void
+netk_tasklist_window_changed_geometry (NetkWindow   *window,
+                                       NetkTasklist *tasklist)
+{
+    NetkTask *win_task;
+    gboolean show;
+    if (tasklist->priv->idle_callback_tag != 0)
     {
-        return;                                                                     
+        return;
     }
     
-    /*                                                                            
-     * We want to re-generate the task list if                                    
-     * the window is shown but shouldn't be or                                    
-     * the window isn't shown but should be.                                      
-     */                                                                           
+    /*
+     * We want to re-generate the task list if
+     * the window is shown but shouldn't be or
+     * the window isn't shown but should be.
+     */
 
-    win_task = g_hash_table_lookup (tasklist->priv->win_hash, window);            
-    show = netk_tasklist_include_window(tasklist, window)
-        && !(netk_window_get_state (window) & NETK_WINDOW_STATE_SKIP_TASKLIST);                        
-    if ((win_task == NULL && !show) || (win_task != NULL && show))                
+    win_task = g_hash_table_lookup (tasklist->priv->win_hash, window);
+    show = netk_tasklist_include_window(tasklist, window);
+    if ((win_task == NULL && !show) || (win_task != NULL && show))
     {
-        return;                                                                     
+        return;
     }
     
-    /* Don't keep any stale references */                                         
+    /* Don't keep any stale references */
     gtk_widget_queue_draw (GTK_WIDGET (tasklist));
     tasklist->priv->idle_callback_tag = 
                 g_idle_add (do_netk_tasklist_update_lists, tasklist);
-}                                                                               
+}
 
 static void
 netk_tasklist_connect_window (NetkTasklist * tasklist, NetkWindow * window)
@@ -1711,10 +1698,11 @@ netk_tasklist_activate_task_window (NetkTask * task)
 
         active_ws = netk_screen_get_active_workspace (tasklist->priv->screen);
         window_ws = netk_window_get_workspace (task->window);
-        if (window_ws && active_ws != window_ws
-            && !tasklist->priv->switch_workspace_on_unminimize)
+        if ((window_ws && active_ws != window_ws)
+            && (!tasklist->priv->switch_workspace_on_unminimize))
+        {
             netk_workspace_activate (window_ws);
-
+        }
         netk_window_activate (task->window);
     }
     else
@@ -1731,7 +1719,9 @@ netk_tasklist_activate_task_window (NetkTask * task)
 
             window_ws = netk_window_get_workspace (task->window);
             if (window_ws)
+            {
                 netk_workspace_activate (window_ws);
+            }
 
             netk_window_activate (task->window);
         }
@@ -2057,9 +2047,7 @@ netk_task_state_changed (NetkWindow * window, NetkWindowState changed_mask,
             if(new_state & NETK_WINDOW_STATE_URGENT)
             {
                 win_task->blink_timeout_id = 
-                                       g_timeout_add (500, 
-                                                      urgent_timeout_callback, 
-                                                      win_task);
+                    g_timeout_add (500, urgent_timeout_callback, win_task);
             }
             else 
             {
@@ -2255,9 +2243,14 @@ netk_task_create_widgets (NetkTask * task)
 
     if (task->tasklist->priv->show_label) {
         task->label = gtk_label_new (text);
+        gtk_misc_set_alignment (GTK_MISC (task->label), 0.5, 0.5);
+        if ((gtk_major_version == 2 && gtk_minor_version >= 6) || gtk_major_version > 2)
+        {
+            g_object_set (G_OBJECT (task->label), "ellipsize", NETK_ELLIPSIZE_END, NULL);
+        }
         gtk_widget_show (task->label);
         gtk_table_attach (GTK_TABLE (table),
-                      task->label, 1, 2, 0, 1, GTK_EXPAND, GTK_EXPAND, 0, 0);
+                      task->label, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, GTK_EXPAND, 0, 0);
     }
 
     gtk_container_add (GTK_CONTAINER (task->button), table);
