@@ -1,4 +1,4 @@
-/* $Id: splash.c,v 1.7 2004/09/19 14:47:02 jasper Exp $ */
+/* $Id: splash.c 18977 2005-11-30 20:24:01Z benny $ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -38,8 +38,8 @@
 #include <libxfcegui4/libxfcegui4.h>
 #include <xfce-mcs-manager/manager-plugin.h>
 
-#include <xfce4-session/xfsm-splash-engine.h>
-#include <xfce4-session/xfsm-util.h>
+#include <libxfsm/xfsm-util.h>
+#include <libxfsm/xfsm-splash-engine.h>
 
 #include <settings/splash/module.h>
 #include <settings/splash/nopreview.h>
@@ -367,7 +367,7 @@ splash_run (McsPlugin *plugin)
 
   /* query kiosk settings */
   kiosk = xfce_kiosk_new ("xfce4-session");
-  kiosk_can_splash = xfce_kiosk_query (kiosk, "Splash");
+  kiosk_can_splash = xfce_kiosk_query (kiosk, "Splash") || xfce_kiosk_query (kiosk, "CustomizeSplash");
   xfce_kiosk_free (kiosk);
 
   /* load config */
